package name.milesparker.epi.plague;

import org.eclipse.amp.agf.IGraphicsAdapter;
import org.eclipse.amp.agf.GraphicsAdapter;
import org.eclipse.amp.agf.gef.IFigureProvider;
import org.eclipse.core.runtime.IAdapterFactory;

import org.eclipse.amp.agf.gef.IFigureProvider;
import org.eclipse.jface.viewers.IColorProvider;
import org.eclipse.jface.viewers.ILabelProvider;

/**
 * Epidemic Java Implementation.
 * A simple model of epidemic dynamics. We define individual agents with different disease states. The progression of disease moves from succeptible to exposed to infectious (either symptomatic or asymptomatic) to some outcome state. Agents can transmit infection to their immediate neighbors.

 For some background on the general approach, see Joshua M. Epstein. "Generative Social Science". 2006. Princeton. 

 Copyright 2009, Miles Parker. Released under Eclipse Public License. All uses should be attributed.
 * Generated by AMF for model: EpidemicPlague.metaabm in project: org.eclipse.amp.amf.examples.escape 
 */
public class EpidemicGraphicsAdapter extends GraphicsAdapter {

	private static IGraphicsAdapter singleton;

	IColorProvider[] epidemicColorProviders = new IColorProvider[]{};
	IColorProvider[] individualColorProviders = new IColorProvider[]{IndividualStyle2DColorProvider
			.getDefault()};
	IColorProvider[] fleaColorProviders = new IColorProvider[]{FleaStyle2DColorProvider
			.getDefault()};
	IColorProvider[] ratColorProviders = new IColorProvider[]{RatStyle2DColorProvider
			.getDefault()};

	IFigureProvider[] epidemicFigureProviders = new IFigureProvider[]{};
	IFigureProvider[] individualFigureProviders = new IFigureProvider[]{IndividualStyle2DFigureProvider
			.getDefault()};
	IFigureProvider[] fleaFigureProviders = new IFigureProvider[]{FleaStyle2DFigureProvider
			.getDefault()};
	IFigureProvider[] ratFigureProviders = new IFigureProvider[]{RatStyle2DFigureProvider
			.getDefault()};

	/**
	 * @param type
	 * @return
	 * @see org.eclipse.amp.agf.IGraphicsAdapter#getColorsForClass(java.lang.Class)
	 */
	public IColorProvider[] getColorsForClass(Class type) {
		if (type == Epidemic.class) {
			return epidemicColorProviders;
		}
		if (type == Individual.class) {
			return individualColorProviders;
		}
		if (type == Flea.class) {
			return fleaColorProviders;
		}
		if (type == Rat.class) {
			return ratColorProviders;
		}
		return null;
	}

	/**
	 * @param type
	 * @return
	 * @see org.eclipse.amp.agf.IGraphicsAdapter#getFiguresForClass(java.lang.Class)
	 */
	public IFigureProvider[] getFiguresForClass(Class type) {
		if (type == Epidemic.class) {
			return epidemicFigureProviders;
		}
		if (type == Individual.class) {
			return individualFigureProviders;
		}
		if (type == Flea.class) {
			return fleaFigureProviders;
		}
		if (type == Rat.class) {
			return ratFigureProviders;
		}
		return null;
	}

	/**
	 * @param type
	 * @return
	 * @see org.eclipse.amp.agf.IGraphicsAdapter#getLabelsForClass(java.lang.Class)
	 */
	public ILabelProvider[] getLabelsForClass(Class type) {
		return null;
	}

	public static IGraphicsAdapter getDefault() {
		if (singleton == null) {
			singleton = new EpidemicGraphicsAdapter();
		}
		return singleton;
	}
}
