package name.milesparker.epi.region;

import java.util.List;

import org.eclipse.core.runtime.IAdapterFactory;
import org.eclipse.amp.agf.IGraphicsAdapted;
import org.eclipse.amp.agf.IGraphicsAdapter;

import org.eclipse.swt.graphics.Color;
import org.eclipse.jface.viewers.IColorProvider;

import org.ascape.model.Agent;
import org.ascape.model.Cell;
import org.ascape.model.CellOccupant;
import org.ascape.model.HostCell;
import org.ascape.model.LocatedAgent;
import org.ascape.model.Scape;
import org.ascape.model.event.ScapeEvent;
import org.ascape.model.rule.Rule;
import org.ascape.model.rule.ExecuteThenUpdate;
import org.ascape.model.space.CollectionSpace;
import org.ascape.model.space.Coordinate;
import org.ascape.model.space.Coordinate2DDiscrete;
import org.ascape.model.space.Graph;
import org.ascape.model.space.Location;
import org.ascape.runtime.NonGraphicRunner;
import org.ascape.util.Conditional;
import org.ascape.util.data.DataPoint;
import org.ascape.util.data.DataPointConcrete;
import org.ascape.util.vis.ColorFeature;
import org.ascape.util.vis.ColorFeatureConcrete;
import org.ascape.view.vis.ChartView;
import org.ascape.view.vis.GEFView;
import org.ascape.view.vis.GraphView;

/**
 * City Java Implementation.
 * A simple model of epidemic dynamics. We define individual agents with different disease states. The progression of disease moves from succeptible to exposed to infectious (either symptomatic or asymptomatic) to some outcome state. Agents can transmit infection to their immediate neighbors.

 For some background on the general approach, see Joshua M. Epstein. "Generative Social Science". 2006. Princeton. 

 Copyright 2009, Miles Parker. Released under Eclipse Public License. All uses should be attributed.
 * Generated by AMF for model: EpidemicRegional.metaabm in project: org.eclipse.amp.amf.examples.escape 
 */
public class City extends Scape implements IGraphicsAdapted {

	/**
	 * The number of individuals to create.
	 */
	private int individualCount = 300;

	/**
	 * 
	 */
	private int periodsperDay = 24;

	/**
	 * The number of locations to create.
	 */
	private int locationCount = 0;

	/**
	 * 
	 */
	private CommunityInfectionStatusEnum communityInfectionStatus = CommunityInfectionStatusEnum.noInfections;

	/**
	 * The size of each dimension.
	 */
	private int[] mapDimensions = new int[2];

	/**
	 * The horizontal extent of the space.
	 */
	private int mapWidth = 20;

	/**
	 * The vertical extent of the space.
	 */
	private int mapHeight = 50;

	/**
	 * 
	 */
	private Scape map = null;

	/**
	 * Constructs a new City.
	 */
	public City() {
	}

	//todo, make this a useful value for evaluating compatibility of different versions of generated classes
	private static final long serialVersionUID = 89989998L;

	private static long nextUniqueID;

	private long uniqueID;

	public long getUID() {
		if (uniqueID == 0) {
			uniqueID = nextUniqueID++;
		}
		return uniqueID;
	}

	/**
	 * Clones the agent, ensuring that a unique id is assigned.
	 */
	public Object clone() {
		try {
			City clone = (City) super.clone();
			clone.uniqueID = 0;
			return clone;
		} catch (Exception e) {
			throw new RuntimeException("Unexpected cloning exception: " + e);
		}
	}

	org.ascape.model.Scape individualScape;

	/**
	 * Create an instance of Individual that will be used to populate the individualScape.
	 * Overide to customize the prototype, for example to change the views created for a member scape.
	 */
	protected Individual createIndividualPrototype() {
		return new Individual();
	}

	static int next_vm_id;

	int vm_unique_id;

	/**
	 * Creates the members of City. 
	 */
	public void createScape() {
		super.createScape();
		vm_unique_id = next_vm_id;
		setName("City " + vm_unique_id);
		next_vm_id++;
		setAutoCreate(false);
		setPrototypeAgent(new org.ascape.model.Scape());
		Individual individualProto = createIndividualPrototype();
		individualScape = new Scape();
		individualScape.setName("Individuals");
		individualScape.setPrototypeAgent(individualProto);
		individualScape.setExecutionOrder(Scape.RULE_ORDER);

		individualScape.setSize(individualCount);
		map = new Scape(new org.ascape.model.space.Array2DMoore());
		map.setExecutionOrder(Scape.RULE_ORDER);
		map.setPrototypeAgent(new Place());
		map.setExtent(new org.ascape.model.space.Coordinate2DDiscrete(
				getMapWidth(), getMapHeight()));

		map.setName("Maps");
		((CollectionSpace) map.getSpace()).setPeriodic(false);
		add(map);
		org.ascape.model.Scape placeScape = map;

		add(individualScape);

		individualProto.setHostScape(map);
		map.getRules().clear();

		individualScape
				.addStatCollector(new org.ascape.util.data.StatCollectorCond(
						getName() + " Population") {
					private static final long serialVersionUID = 684614444640209893L;

					@SuppressWarnings("unused")
					public final boolean meetsCondition(Object object) {
						return true;
					}
				});
		individualScape
				.addStatCollector(new org.ascape.util.data.StatCollectorCond(
						getName() + " Individual Susceptible Status") {
					private static final long serialVersionUID = 6846144446402098985L;
					public final boolean meetsCondition(Object individual) {
						return StatusEnum.susceptible == ((Individual) individual)
								.getStatus();
					}
				});

		individualScape
				.addStatCollector(new org.ascape.util.data.StatCollectorCond(
						getName() + " Individual Exposed Status") {
					private static final long serialVersionUID = 6846144446402098985L;
					public final boolean meetsCondition(Object individual) {
						return StatusEnum.exposed == ((Individual) individual)
								.getStatus();
					}
				});

		individualScape
				.addStatCollector(new org.ascape.util.data.StatCollectorCond(
						getName() + " Individual Asymptom Infectious Status") {
					private static final long serialVersionUID = 6846144446402098985L;
					public final boolean meetsCondition(Object individual) {
						return StatusEnum.asymptomInfectious == ((Individual) individual)
								.getStatus();
					}
				});

		individualScape
				.addStatCollector(new org.ascape.util.data.StatCollectorCond(
						getName() + " Individual Symptom Infectious Status") {
					private static final long serialVersionUID = 6846144446402098985L;
					public final boolean meetsCondition(Object individual) {
						return StatusEnum.symptomInfectious == ((Individual) individual)
								.getStatus();
					}
				});

		individualScape
				.addStatCollector(new org.ascape.util.data.StatCollectorCond(
						getName() + " Individual Recovered Status") {
					private static final long serialVersionUID = 6846144446402098985L;
					public final boolean meetsCondition(Object individual) {
						return StatusEnum.recovered == ((Individual) individual)
								.getStatus();
					}
				});

		individualScape
				.addStatCollector(new org.ascape.util.data.StatCollectorCond(
						getName() + " Individual Dead Status") {
					private static final long serialVersionUID = 6846144446402098985L;
					public final boolean meetsCondition(Object individual) {
						return StatusEnum.dead == ((Individual) individual)
								.getStatus();
					}
				});

		individualScape
				.addStatCollector(new org.ascape.util.data.StatCollectorCSAMM(
						getName() + " Individual Exposure End Period") {
					private static final long serialVersionUID = 6846144446402098985L;
					public final double getValue(Object individual) {
						return ((Individual) individual).getExposureEndPeriod();
					}
				});

		individualScape
				.addStatCollector(new org.ascape.util.data.StatCollectorCSAMM(
						getName()
								+ " Individual Contact Transmission Probability") {
					private static final long serialVersionUID = 6846144446402098985L;
					public final double getValue(Object individual) {
						return ((Individual) individual)
								.getContactTransmissionProbability();
					}
				});

		individualScape.addInitialRule(new Rule("Initialize Location") {
			private static final long serialVersionUID = 6846144446402098982L;
			public void execute(Agent a) {
				((name.milesparker.epi.region.Individual) a)
						.initializeLocation();
			}
		});
		individualScape.addRule(new Rule("Movement") {
			private static final long serialVersionUID = 6846144446402098981L;
			public void execute(Agent a) {
				((name.milesparker.epi.region.Individual) a).movement();
			}
		});

		individualScape.addInitialRule(new Rule("Initialize State") {
			private static final long serialVersionUID = 6846144446402098982L;
			public void execute(Agent a) {
				((name.milesparker.epi.region.Individual) a).initializeState();
			}
		});
		individualScape.addRule(new Rule("Transmission") {
			private static final long serialVersionUID = 6846144446402098981L;
			public void execute(Agent a) {
				((name.milesparker.epi.region.Individual) a).transmission();
			}
		});
		individualScape.addRule(new Rule("Progression") {
			private static final long serialVersionUID = 6846144446402098981L;
			public void execute(Agent a) {
				((name.milesparker.epi.region.Individual) a).progression();
			}
		});
		individualScape.addRule(new Rule("Migrate") {
			private static final long serialVersionUID = 6846144446402098981L;
			public void execute(Agent a) {
				((name.milesparker.epi.region.Individual) a).migrate();
			}
		});

	}

	/**
	 * Creates UI views for City.
	 */
	public void createGraphicViews() {
		super.createGraphicViews();
		addView(new GEFView());
	}

	public void scapeSetup(ScapeEvent scapeEvent) {
		//Set the size of the scape populations, e.g. for mutable scapes that need to have their size reset

		individualScape.setSize(individualCount);
	}

	/**
	 * Returns the Scape containing Individuals.
	 */
	public org.ascape.model.Scape getIndividualScape() {
		return individualScape;
	}

	public IGraphicsAdapter getGraphicsAdapter() {
		return CityGraphicsAdapter.getDefault();
	}

	private Color getInferredChartColor(Object agent,
			IColorProvider colorProvider) {
		Color color = colorProvider.getForeground(agent);
		if (color == ColorFeature.BLACK) {
			color = ColorFeatureConcrete.createHSB(
					(float) java.lang.Math.random() * 360f, 1.0f, 0.8f);
		}
		return color;
	}

	public static void main(String[] args) {
		(new NonGraphicRunner())
				.openInstance("name.milesparker.epi.region.City");
	}
	public Region getRegion() {
		return (Region) getScape().getScape();
	}

	/**
	 * Setup Location Initialization. Executed once at the beginning of each model run.
	 */
	public void setupLocation() {
		RegionalLocation cityLocation = (RegionalLocation) ((org.ascape.model.space.Discrete) getRegion()
				.getRegionalMap().getSpace()).findRandomAvailable();
		if (cityLocation != null) {
			if (getHostScape() != ((Agent) cityLocation).getScape()) {
				die();
				getRegion().getCityScape().add(this);
			}
			moveTo(cityLocation);
		}
	}
	/**
	 * Gets the Individual Count property for City.
	 * @return The number of individuals to create.
	 */
	public int getIndividualCount() {
		return individualCount;
	}

	/**
	 * Sets the Individual Count property for City.
	 * The number of individuals to create.
	 * @param _individualCount the new Individual Count value
	 */
	public void setIndividualCount(int _individualCount) {
		individualCount = _individualCount;
	}

	/**
	 * Gets the Periods per Day property for City.
	 * @return 
	 */
	public int getPeriodsperDay() {
		return periodsperDay;
	}

	/**
	 * Sets the Periods per Day property for City.
	 * 
	 * @param _periodsperDay the new Periods per Day value
	 */
	public void setPeriodsperDay(int _periodsperDay) {
		periodsperDay = _periodsperDay;
	}

	/**
	 * Gets the Location Count property for City.
	 * @return The number of locations to create.
	 */
	public int getLocationCount() {
		return locationCount;
	}

	/**
	 * Sets the Location Count property for City.
	 * The number of locations to create.
	 * @param _locationCount the new Location Count value
	 */
	public void setLocationCount(int _locationCount) {
		locationCount = _locationCount;
	}

	/**
	 * Gets the Community Infection Status property for City.
	 * @return 
	 */
	public CommunityInfectionStatusEnum getCommunityInfectionStatus() {
		return communityInfectionStatus;
	}

	/**
	 * Sets the Community Infection Status property for City.
	 * 
	 * @param _communityInfectionStatus the new Community Infection Status value
	 */
	public void setCommunityInfectionStatus(
			CommunityInfectionStatusEnum _communityInfectionStatus) {
		communityInfectionStatus = _communityInfectionStatus;
	}

	/**
	 * Gets the Dimensions property for Map.
	 * @return The size of each dimension.
	 */
	public int[] getMapDimensions() {
		return mapDimensions;
	}

	/**
	 * Sets the Dimensions property for Map.
	 * The size of each dimension.
	 * @param _mapDimensions the new Dimensions value
	 */
	public void setMapDimensions(int[] _mapDimensions) {
		mapDimensions = _mapDimensions;
	}

	/**
	 * Gets the Width property for Map.
	 * @return The horizontal extent of the space.
	 */
	public int getMapWidth() {
		return mapWidth;
	}

	/**
	 * Sets the Width property for Map.
	 * The horizontal extent of the space.
	 * @param _mapWidth the new Width value
	 */
	public void setMapWidth(int _mapWidth) {
		mapWidth = _mapWidth;
	}

	/**
	 * Gets the Height property for Map.
	 * @return The vertical extent of the space.
	 */
	public int getMapHeight() {
		return mapHeight;
	}

	/**
	 * Sets the Height property for Map.
	 * The vertical extent of the space.
	 * @param _mapHeight the new Height value
	 */
	public void setMapHeight(int _mapHeight) {
		mapHeight = _mapHeight;
	}

	/**
	 * Gets the Map property for City.
	 * @return 
	 */
	public Scape getMap() {
		return map;
	}

	/**
	 * Sets the Map property for City.
	 * 
	 * @param _map the new Map value
	 */
	public void setMap(Scape _map) {
		map = _map;
	}

	public String getName() {
		if (name == null) {
			return "City " + getUID();
		} else {
			return name;
		}
	}
}
