package name.milesparker.epi.region;

import java.util.List;

import org.eclipse.core.runtime.IAdapterFactory;
import org.eclipse.amp.agf.IGraphicsAdapted;
import org.eclipse.amp.agf.IGraphicsAdapter;

import org.eclipse.swt.graphics.Color;
import org.eclipse.jface.viewers.IColorProvider;

import org.ascape.model.Agent;
import org.ascape.model.Cell;
import org.ascape.model.CellOccupant;
import org.ascape.model.HostCell;
import org.ascape.model.LocatedAgent;
import org.ascape.model.Scape;
import org.ascape.model.event.ScapeEvent;
import org.ascape.model.rule.Rule;
import org.ascape.model.rule.ExecuteThenUpdate;
import org.ascape.model.space.CollectionSpace;
import org.ascape.model.space.Coordinate;
import org.ascape.model.space.Coordinate2DDiscrete;
import org.ascape.model.space.Graph;
import org.ascape.model.space.Location;
import org.ascape.runtime.NonGraphicRunner;
import org.ascape.util.Conditional;
import org.ascape.util.data.DataPoint;
import org.ascape.util.data.DataPointConcrete;
import org.ascape.util.vis.ColorFeature;
import org.ascape.util.vis.ColorFeatureConcrete;
import org.ascape.view.vis.ChartView;
import org.ascape.view.vis.GEFView;
import org.ascape.view.vis.GraphView;

/**
 * Individual Java Implementation.
 * 
 * Generated by AMF for model: EpidemicRegional.metaabm in project: org.eclipse.amp.amf.examples.escape 
 */
public class Individual extends CellOccupant {

	/**
	 * 
	 */
	private StatusEnum status = StatusEnum.susceptible;

	/**
	 * 
	 */
	private int exposureEndPeriod = 0;

	/**
	 * 
	 */
	private int asymptomaticEndPeriod = 0;

	/**
	 * 
	 */
	private int outcomePeriod = 0;

	/**
	 * 
	 */
	private double contactTransmissionProbability = 0.0;

	/**
	 * Constructs a new Individual.
	 */
	public Individual() {
	}
	//todo, make this a useful value for evaluating compatibility of different versions of generated classes
	private static final long serialVersionUID = 89989998L;

	private static long nextUniqueID;

	private long uniqueID;

	public long getUID() {
		if (uniqueID == 0) {
			uniqueID = nextUniqueID++;
		}
		return uniqueID;
	}

	/**
	 * Clones the agent, ensuring that a unique id is assigned.
	 */
	public Object clone() {
		try {
			Individual clone = (Individual) super.clone();
			clone.uniqueID = 0;
			return clone;
		} catch (Exception e) {
			throw new RuntimeException("Unexpected cloning exception: " + e);
		}
	}

	public City getCity() {
		return (City) getScape().getScape();
	}

	/**
	 * Initialize Location Initialization. Executed once at the beginning of each model run.
	 */
	public void initializeLocation() {
		Place individual = (Place) ((org.ascape.model.space.Discrete) getCity()
				.getMap().getSpace()).findRandomAvailable();
		if (individual != null) {
			if (getHostScape() != ((Agent) individual).getScape()) {
				die();
				getCity().getIndividualScape().add(this);
			}
			moveTo(individual);
		}
	}

	/**
	 * Movement Rule. Executed every period.
	 */
	public void movement() {
		double movementDraw = getRandom().nextDouble();
		if ((getStatus() != StatusEnum.dead
				&& movementDraw < getCity().getRegion()
						.getMovementProbability()
				&& getStatus() != StatusEnum.symptomInfectious && movementDraw < getCity()
				.getRegion().getMovementProbability())) {
			Place neighboringLocation = (Place) ((org.ascape.model.space.Discrete) getCity()
					.getMap().getSpace())
					.findRandomAvailableNeighbor(((org.ascape.model.CellOccupant) this)
							.getHostCell());
			if (neighboringLocation != null) {
				if (getHostScape() != ((Agent) neighboringLocation).getScape()) {
					die();
					getCity().getIndividualScape().add(this);
				}
				moveTo(neighboringLocation);
			}
		}
	}

	/**
	 * Initialize State Initialization. Executed once at the beginning of each model run.
	 */
	public void initializeState() {
		double individualTransmissionRate = randomInRange(getCity().getRegion()
				.getMinContactTransmissionProbability(), getCity().getRegion()
				.getMaxContactTransmissionProbability());
		setContactTransmissionProbability(individualTransmissionRate);
		if (getCity().getCommunityInfectionStatus() == CommunityInfectionStatusEnum.originCity) {
			double infectionDraw = getRandom().nextDouble();
			if (infectionDraw < getCity().getRegion()
					.getInitialInfectionProbability()) {
				setStatus(StatusEnum.exposed);
			}
		}
		if (!getCity().getRegion().isInitialCityAssigned()) {
			getCity().getRegion().setInitialCityAssigned(true);
			getCity().setCommunityInfectionStatus(
					CommunityInfectionStatusEnum.originCity);
			if (getCity().getCommunityInfectionStatus() == CommunityInfectionStatusEnum.originCity) {
				double infectionDraw = getRandom().nextDouble();
				if (infectionDraw < getCity().getRegion()
						.getInitialInfectionProbability()) {
					setStatus(StatusEnum.exposed);
				}
			}
		}
	}

	/**
	 * Transmission Rule. Executed every period.
	 */
	public void transmission() {
		if ((getStatus() == StatusEnum.symptomInfectious || getStatus() == StatusEnum.asymptomInfectious)) {
			Conditional vulnerableCondition = new Conditional() {
				private static final long serialVersionUID = 6846144446402098985L;
				public boolean meetsCondition(Object vulnerableCell) {
					if (vulnerableCell instanceof Individual) {
						return true;
					} else {
						return false;
					}
				}
			};
			Individual vulnerable = (Individual) ((org.ascape.model.space.Discrete) getCity()
					.getMap().getSpace()).findRandomNeighbor(this,
					vulnerableCondition);
			if (vulnerable != null) {
				if (vulnerable.getStatus() == StatusEnum.susceptible) {
					double transmissionDraw = getRandom().nextDouble();
					if (transmissionDraw < vulnerable
							.getContactTransmissionProbability()) {
						vulnerable.setStatus(StatusEnum.exposed);
					}
				}
			}
		}
	}

	/**
	 * Progression Rule. Executed every period.
	 */
	public void progression() {
		int progressionPeriod = getScape().getPeriod();
		if ((getExposureEndPeriod() != getAsymptomaticEndPeriod() && progressionPeriod == getExposureEndPeriod())) {
			setStatus(StatusEnum.asymptomInfectious);
		}
		if (progressionPeriod == getOutcomePeriod()) {
			double mortalityDraw = getRandom().nextDouble();
			if (mortalityDraw < getCity().getRegion().getCaseMortalityRate()) {
				setStatus(StatusEnum.dead);
			}
			if (!(mortalityDraw < getCity().getRegion().getCaseMortalityRate())) {
				setStatus(StatusEnum.recovered);
			}
		}
		if ((progressionPeriod == getExposureEndPeriod() && getExposureEndPeriod() == getAsymptomaticEndPeriod())) {
			setStatus(StatusEnum.symptomInfectious);
		}
		if ((progressionPeriod == getAsymptomaticEndPeriod() && getExposureEndPeriod() != getAsymptomaticEndPeriod())) {
			setStatus(StatusEnum.symptomInfectious);
			getCity().setCommunityInfectionStatus(
					CommunityInfectionStatusEnum.infectionsObserved);
		}
	}

	/**
	 * Watch for changes in Status.
	 */
	public void exposure() {
		if (getStatus() == StatusEnum.exposed) {
			int exposureNow = getScape().getPeriod();
			int exposureLength = randomInRange(getCity().getRegion()
					.getMinPeriodExposure(), getCity().getRegion()
					.getMaxPeriodExposure());
			int endExposure = exposureNow + exposureLength;
			setExposureEndPeriod(endExposure);
			int asymptomaticLength = randomInRange(getCity().getRegion()
					.getMinPeriodAsymptomInfection(), getCity().getRegion()
					.getMaxPeriodAsymptomInfection());
			int endAsymptomatic = endExposure + asymptomaticLength;
			setAsymptomaticEndPeriod(endAsymptomatic);
			int symptomLength = randomInRange(getCity().getRegion()
					.getMinPeriodSymptomInfection(), getCity().getRegion()
					.getMaxPeriodSymptomInfection());
			int endInfection = endAsymptomatic + symptomLength;
			setOutcomePeriod(endInfection);
		}
	}

	/**
	 * Migrate Rule. Executed every period.
	 */
	public void migrate() {
		double migrateRandomUnit = getRandom().nextDouble();
		if (migrateRandomUnit < getCity().getRegion().getMigrationProbability()) {
			Conditional neighboringCityCondition = new Conditional() {
				private static final long serialVersionUID = 6846144446402098985L;
				public boolean meetsCondition(Object neighboringCityCell) {
					if (neighboringCityCell instanceof City) {
						return true;
					} else {
						return false;
					}
				}
			};
			City neighboringCity = (City) ((org.ascape.model.space.Discrete) getCity()
					.getRegion().getRegionalMap().getSpace())
					.findRandomNeighbor(getCity(), neighboringCityCondition);
			if (neighboringCity != null) {
				Place newLocation = (Place) ((org.ascape.model.space.Discrete) neighboringCity
						.getMap().getSpace()).findRandomAvailable();
				if (newLocation != null) {
					if (getHostScape() != ((Agent) newLocation).getScape()) {
						die();
						getCity().getIndividualScape().add(this);
					}
					moveTo(newLocation);
				}
			}
		}
	}
	/**
	 * Gets the Status property for Individual.
	 * @return 
	 */
	public StatusEnum getStatus() {
		return status;
	}

	/**
	 * Sets the Status property for Individual.
	 * 
	 * @param _status the new Status value
	 */
	public void setStatus(StatusEnum _status) {
		status = _status;
		exposure();
	}

	/**
	 * Gets the Exposure End Period property for Individual.
	 * @return 
	 */
	public int getExposureEndPeriod() {
		return exposureEndPeriod;
	}

	/**
	 * Sets the Exposure End Period property for Individual.
	 * 
	 * @param _exposureEndPeriod the new Exposure End Period value
	 */
	public void setExposureEndPeriod(int _exposureEndPeriod) {
		exposureEndPeriod = _exposureEndPeriod;
	}

	/**
	 * Gets the Asymptomatic End Period property for Individual.
	 * @return 
	 */
	public int getAsymptomaticEndPeriod() {
		return asymptomaticEndPeriod;
	}

	/**
	 * Sets the Asymptomatic End Period property for Individual.
	 * 
	 * @param _asymptomaticEndPeriod the new Asymptomatic End Period value
	 */
	public void setAsymptomaticEndPeriod(int _asymptomaticEndPeriod) {
		asymptomaticEndPeriod = _asymptomaticEndPeriod;
	}

	/**
	 * Gets the Outcome Period property for Individual.
	 * @return 
	 */
	public int getOutcomePeriod() {
		return outcomePeriod;
	}

	/**
	 * Sets the Outcome Period property for Individual.
	 * 
	 * @param _outcomePeriod the new Outcome Period value
	 */
	public void setOutcomePeriod(int _outcomePeriod) {
		outcomePeriod = _outcomePeriod;
	}

	/**
	 * Gets the Contact Transmission Probability property for Individual.
	 * @return 
	 */
	public double getContactTransmissionProbability() {
		return contactTransmissionProbability;
	}

	/**
	 * Sets the Contact Transmission Probability property for Individual.
	 * 
	 * @param _contactTransmissionProbability the new Contact Transmission Probability value
	 */
	public void setContactTransmissionProbability(
			double _contactTransmissionProbability) {
		contactTransmissionProbability = _contactTransmissionProbability;
	}

	public String getName() {
		if (name == null) {
			return "Individual " + getUID();
		} else {
			return name;
		}
	}
}
