package name.milesparker.epi.region;

import java.util.List;

import org.eclipse.core.runtime.IAdapterFactory;
import org.eclipse.amp.agf.IGraphicsAdapted;
import org.eclipse.amp.agf.IGraphicsAdapter;

import org.eclipse.swt.graphics.Color;
import org.eclipse.jface.viewers.IColorProvider;

import org.ascape.model.Agent;
import org.ascape.model.Cell;
import org.ascape.model.CellOccupant;
import org.ascape.model.HostCell;
import org.ascape.model.LocatedAgent;
import org.ascape.model.Scape;
import org.ascape.model.event.ScapeEvent;
import org.ascape.model.rule.Rule;
import org.ascape.model.rule.ExecuteThenUpdate;
import org.ascape.model.space.CollectionSpace;
import org.ascape.model.space.Coordinate;
import org.ascape.model.space.Coordinate2DDiscrete;
import org.ascape.model.space.Graph;
import org.ascape.model.space.Location;
import org.ascape.runtime.NonGraphicRunner;
import org.ascape.util.Conditional;
import org.ascape.util.data.DataPoint;
import org.ascape.util.data.DataPointConcrete;
import org.ascape.util.vis.ColorFeature;
import org.ascape.util.vis.ColorFeatureConcrete;
import org.ascape.view.vis.ChartView;
import org.ascape.view.vis.GEFView;
import org.ascape.view.vis.GraphView;

/**
 * Region Java Implementation.
 * 
 * Generated by AMF for model: EpidemicRegional.metaabm in project: org.eclipse.amp.amf.examples.escape 
 */
public class Region extends Scape implements IGraphicsAdapted {

	/**
	 * The number of citys to create.
	 */
	private int cityCount = 4;

	/**
	 * The probability that any given individual will be infected at the beginning of the model run. For example, if this value is .15, and their are 100 individuals in the model, roughly 15 of those individuals will be infected at time 0.
	 */
	private double initialInfectionProbability = .05;

	/**
	 * The probability that proximity to a single neighbor for a single period will result in transmission of infection.
	 */
	private double minContactTransmissionProbability = .08;

	/**
	 * The probability that proximity to a single neighbor for a single period will result in transmission of infection.
	 */
	private double maxContactTransmissionProbability = .16;

	/**
	 * The probability that death will occur for a given individual as a result of infection.
	 */
	private double caseMortalityRate = .05;

	/**
	 * Minimum time from initial exposure to infectious asymptomatic or symptomatic state. 0 or greater.
	 */
	private int minPeriodExposure = 12;

	/**
	 * Maximum time from initial exposure to infectious asymptomatic or symptomatic state. 0 or greater.
	 */
	private int maxPeriodExposure = 48;

	/**
	 * Maximum time spent infected without showing symptoms. 0 or greater.
	 */
	private int minPeriodAsymptomInfection = 24;

	/**
	 * Maximum time spent infected without showing symptoms. May be zero!
	 */
	private int maxPeriodAsymptomInfection = 96;

	/**
	 * Minimum time spent infected with obvious symptoms. 0 or greater.
	 */
	private int minPeriodSymptomInfection = 24;

	/**
	 * Maximum time spent infected with obvious symptoms. 1 or greater.
	 */
	private int maxPeriodSymptomInfection = 168;

	/**
	 * The liklihood that an agent will move within a given period.
	 */
	private double movementProbability = .5;

	/**
	 * 
	 */
	private double migrationProbability = 0.1;

	/**
	 * The number of regional locations to create.
	 */
	private int regionalLocationCount = 0;

	/**
	 * 
	 */
	private boolean initialCityAssigned = false;

	/**
	 * The size of each dimension.
	 */
	private int[] regionalMapDimensions = new int[2];

	/**
	 * The horizontal extent of the space.
	 */
	private int regionalMapWidth = 2;

	/**
	 * The vertical extent of the space.
	 */
	private int regionalMapHeight = 2;

	/**
	 * 
	 */
	private Scape regionalMap = null;

	/**
	 * Constructs a new Region.
	 */
	public Region() {
	}

	//todo, make this a useful value for evaluating compatibility of different versions of generated classes
	private static final long serialVersionUID = 89989998L;

	private static long nextUniqueID;

	private long uniqueID;

	public long getUID() {
		if (uniqueID == 0) {
			uniqueID = nextUniqueID++;
		}
		return uniqueID;
	}

	/**
	 * Clones the agent, ensuring that a unique id is assigned.
	 */
	public Object clone() {
		try {
			Region clone = (Region) super.clone();
			clone.uniqueID = 0;
			return clone;
		} catch (Exception e) {
			throw new RuntimeException("Unexpected cloning exception: " + e);
		}
	}

	org.ascape.model.Scape cityScape;

	/**
	 * Create an instance of City that will be used to populate the cityScape.
	 * Overide to customize the prototype, for example to change the views created for a member scape.
	 */
	protected City createCityPrototype() {
		return new City();
	}

	static int next_vm_id;

	int vm_unique_id;

	/**
	 * Creates the members of Region. 
	 */
	public void createScape() {
		super.createScape();

		setName("Region");
		City cityProto = createCityPrototype();
		cityScape = new Scape();
		cityScape.setName("Citys");
		cityScape.setPrototypeAgent(cityProto);
		cityScape.setExecutionOrder(Scape.RULE_ORDER);

		cityScape.setSize(cityCount);
		regionalMap = new Scape(new org.ascape.model.space.Array2DMoore());
		regionalMap.setExecutionOrder(Scape.RULE_ORDER);
		regionalMap.setPrototypeAgent(new RegionalLocation());
		regionalMap.setExtent(new org.ascape.model.space.Coordinate2DDiscrete(
				getRegionalMapWidth(), getRegionalMapHeight()));

		regionalMap.setName("Regional Maps");
		((CollectionSpace) regionalMap.getSpace()).setPeriodic(true);
		add(regionalMap);
		org.ascape.model.Scape regionalLocationScape = regionalMap;

		add(cityScape);

		cityProto.setHostScape(regionalMap);
		regionalMap.getRules().clear();

		cityScape.addStatCollector(new org.ascape.util.data.StatCollectorCond(
				getName() + " Population") {
			private static final long serialVersionUID = 684614444640209893L;

			@SuppressWarnings("unused")
			public final boolean meetsCondition(Object object) {
				return true;
			}
		});
		cityScape.setAutoCreate(false);
		cityScape.createScape();

		cityScape.addInitialRule(new Rule("Setup Location") {
			private static final long serialVersionUID = 6846144446402098982L;
			public void execute(Agent a) {
				((name.milesparker.epi.region.City) a).setupLocation();
			}
		});

	}

	/**
	 * Creates UI views for Region.
	 */
	public void createGraphicViews() {
		super.createGraphicViews();
		ChartView chart = new ChartView();

		final Region cityScape = new Region();
		cityScape.setRunner(new NonGraphicRunner());
		City city = new City() {
			public void requestUpdate() {
			}
			public Region getRegion() {
				return cityScape;
			}
		};
		cityScape.add(city);
		HostCell cityHost = new HostCell() {
			public void requestUpdate() {
			}
		};
		cityHost.setOccupant((CellOccupant) city);

		IColorProvider cityStyle2DColorProvider = new CityStyle2DColorProvider();

		addView(chart);
		addView(new GEFView());
	}

	public void scapeSetup(ScapeEvent scapeEvent) {
		//Set the size of the scape populations, e.g. for mutable scapes that need to have their size reset

		cityScape.setSize(cityCount);
	}

	/**
	 * Returns the Scape containing Citys.
	 */
	public org.ascape.model.Scape getCityScape() {
		return cityScape;
	}

	public IGraphicsAdapter getGraphicsAdapter() {
		return RegionGraphicsAdapter.getDefault();
	}

	private Color getInferredChartColor(Object agent,
			IColorProvider colorProvider) {
		Color color = colorProvider.getForeground(agent);
		if (color == ColorFeature.BLACK) {
			color = ColorFeatureConcrete.createHSB(
					(float) java.lang.Math.random() * 360f, 1.0f, 0.8f);
		}
		return color;
	}

	public static void main(String[] args) {
		(new NonGraphicRunner())
				.openInstance("name.milesparker.epi.region.Region");
	}
	/**
	 * Gets the City Count property for Region.
	 * @return The number of citys to create.
	 */
	public int getCityCount() {
		return cityCount;
	}

	/**
	 * Sets the City Count property for Region.
	 * The number of citys to create.
	 * @param _cityCount the new City Count value
	 */
	public void setCityCount(int _cityCount) {
		cityCount = _cityCount;
	}

	/**
	 * Gets the Initial Infection Probability property for Region.
	 * @return The probability that any given individual will be infected at the beginning of the model run. For example, if this value is .15, and their are 100 individuals in the model, roughly 15 of those individuals will be infected at time 0.
	 */
	public double getInitialInfectionProbability() {
		return initialInfectionProbability;
	}

	/**
	 * Sets the Initial Infection Probability property for Region.
	 * The probability that any given individual will be infected at the beginning of the model run. For example, if this value is .15, and their are 100 individuals in the model, roughly 15 of those individuals will be infected at time 0.
	 * @param _initialInfectionProbability the new Initial Infection Probability value
	 */
	public void setInitialInfectionProbability(
			double _initialInfectionProbability) {
		initialInfectionProbability = _initialInfectionProbability;
	}

	/**
	 * Gets the Min Contact Transmission Probability property for Region.
	 * @return The probability that proximity to a single neighbor for a single period will result in transmission of infection.
	 */
	public double getMinContactTransmissionProbability() {
		return minContactTransmissionProbability;
	}

	/**
	 * Sets the Min Contact Transmission Probability property for Region.
	 * The probability that proximity to a single neighbor for a single period will result in transmission of infection.
	 * @param _minContactTransmissionProbability the new Min Contact Transmission Probability value
	 */
	public void setMinContactTransmissionProbability(
			double _minContactTransmissionProbability) {
		minContactTransmissionProbability = _minContactTransmissionProbability;
	}

	/**
	 * Gets the Max Contact Transmission Probability property for Region.
	 * @return The probability that proximity to a single neighbor for a single period will result in transmission of infection.
	 */
	public double getMaxContactTransmissionProbability() {
		return maxContactTransmissionProbability;
	}

	/**
	 * Sets the Max Contact Transmission Probability property for Region.
	 * The probability that proximity to a single neighbor for a single period will result in transmission of infection.
	 * @param _maxContactTransmissionProbability the new Max Contact Transmission Probability value
	 */
	public void setMaxContactTransmissionProbability(
			double _maxContactTransmissionProbability) {
		maxContactTransmissionProbability = _maxContactTransmissionProbability;
	}

	/**
	 * Gets the Case Mortality Rate property for Region.
	 * @return The probability that death will occur for a given individual as a result of infection.
	 */
	public double getCaseMortalityRate() {
		return caseMortalityRate;
	}

	/**
	 * Sets the Case Mortality Rate property for Region.
	 * The probability that death will occur for a given individual as a result of infection.
	 * @param _caseMortalityRate the new Case Mortality Rate value
	 */
	public void setCaseMortalityRate(double _caseMortalityRate) {
		caseMortalityRate = _caseMortalityRate;
	}

	/**
	 * Gets the Min Period Exposure property for Region.
	 * @return Minimum time from initial exposure to infectious asymptomatic or symptomatic state. 0 or greater.
	 */
	public int getMinPeriodExposure() {
		return minPeriodExposure;
	}

	/**
	 * Sets the Min Period Exposure property for Region.
	 * Minimum time from initial exposure to infectious asymptomatic or symptomatic state. 0 or greater.
	 * @param _minPeriodExposure the new Min Period Exposure value
	 */
	public void setMinPeriodExposure(int _minPeriodExposure) {
		minPeriodExposure = _minPeriodExposure;
	}

	/**
	 * Gets the Max Period Exposure property for Region.
	 * @return Maximum time from initial exposure to infectious asymptomatic or symptomatic state. 0 or greater.
	 */
	public int getMaxPeriodExposure() {
		return maxPeriodExposure;
	}

	/**
	 * Sets the Max Period Exposure property for Region.
	 * Maximum time from initial exposure to infectious asymptomatic or symptomatic state. 0 or greater.
	 * @param _maxPeriodExposure the new Max Period Exposure value
	 */
	public void setMaxPeriodExposure(int _maxPeriodExposure) {
		maxPeriodExposure = _maxPeriodExposure;
	}

	/**
	 * Gets the Min Period Asymptom Infection property for Region.
	 * @return Maximum time spent infected without showing symptoms. 0 or greater.
	 */
	public int getMinPeriodAsymptomInfection() {
		return minPeriodAsymptomInfection;
	}

	/**
	 * Sets the Min Period Asymptom Infection property for Region.
	 * Maximum time spent infected without showing symptoms. 0 or greater.
	 * @param _minPeriodAsymptomInfection the new Min Period Asymptom Infection value
	 */
	public void setMinPeriodAsymptomInfection(int _minPeriodAsymptomInfection) {
		minPeriodAsymptomInfection = _minPeriodAsymptomInfection;
	}

	/**
	 * Gets the Max Period Asymptom Infection property for Region.
	 * @return Maximum time spent infected without showing symptoms. May be zero!
	 */
	public int getMaxPeriodAsymptomInfection() {
		return maxPeriodAsymptomInfection;
	}

	/**
	 * Sets the Max Period Asymptom Infection property for Region.
	 * Maximum time spent infected without showing symptoms. May be zero!
	 * @param _maxPeriodAsymptomInfection the new Max Period Asymptom Infection value
	 */
	public void setMaxPeriodAsymptomInfection(int _maxPeriodAsymptomInfection) {
		maxPeriodAsymptomInfection = _maxPeriodAsymptomInfection;
	}

	/**
	 * Gets the Min Period Symptom Infection property for Region.
	 * @return Minimum time spent infected with obvious symptoms. 0 or greater.
	 */
	public int getMinPeriodSymptomInfection() {
		return minPeriodSymptomInfection;
	}

	/**
	 * Sets the Min Period Symptom Infection property for Region.
	 * Minimum time spent infected with obvious symptoms. 0 or greater.
	 * @param _minPeriodSymptomInfection the new Min Period Symptom Infection value
	 */
	public void setMinPeriodSymptomInfection(int _minPeriodSymptomInfection) {
		minPeriodSymptomInfection = _minPeriodSymptomInfection;
	}

	/**
	 * Gets the Max Period Symptom Infection property for Region.
	 * @return Maximum time spent infected with obvious symptoms. 1 or greater.
	 */
	public int getMaxPeriodSymptomInfection() {
		return maxPeriodSymptomInfection;
	}

	/**
	 * Sets the Max Period Symptom Infection property for Region.
	 * Maximum time spent infected with obvious symptoms. 1 or greater.
	 * @param _maxPeriodSymptomInfection the new Max Period Symptom Infection value
	 */
	public void setMaxPeriodSymptomInfection(int _maxPeriodSymptomInfection) {
		maxPeriodSymptomInfection = _maxPeriodSymptomInfection;
	}

	/**
	 * Gets the Movement Probability property for Region.
	 * @return The liklihood that an agent will move within a given period.
	 */
	public double getMovementProbability() {
		return movementProbability;
	}

	/**
	 * Sets the Movement Probability property for Region.
	 * The liklihood that an agent will move within a given period.
	 * @param _movementProbability the new Movement Probability value
	 */
	public void setMovementProbability(double _movementProbability) {
		movementProbability = _movementProbability;
	}

	/**
	 * Gets the Migration Probability property for Region.
	 * @return 
	 */
	public double getMigrationProbability() {
		return migrationProbability;
	}

	/**
	 * Sets the Migration Probability property for Region.
	 * 
	 * @param _migrationProbability the new Migration Probability value
	 */
	public void setMigrationProbability(double _migrationProbability) {
		migrationProbability = _migrationProbability;
	}

	/**
	 * Gets the Regional Location Count property for Region.
	 * @return The number of regional locations to create.
	 */
	public int getRegionalLocationCount() {
		return regionalLocationCount;
	}

	/**
	 * Sets the Regional Location Count property for Region.
	 * The number of regional locations to create.
	 * @param _regionalLocationCount the new Regional Location Count value
	 */
	public void setRegionalLocationCount(int _regionalLocationCount) {
		regionalLocationCount = _regionalLocationCount;
	}

	/**
	 * Gets the Initial City Assigned property for Region.
	 * @return 
	 */
	public boolean isInitialCityAssigned() {
		return initialCityAssigned;
	}

	/**
	 * Sets the Initial City Assigned property for Region.
	 * 
	 * @param _initialCityAssigned the new Initial City Assigned value
	 */
	public void setInitialCityAssigned(boolean _initialCityAssigned) {
		initialCityAssigned = _initialCityAssigned;
	}

	/**
	 * Gets the Dimensions property for Regional Map.
	 * @return The size of each dimension.
	 */
	public int[] getRegionalMapDimensions() {
		return regionalMapDimensions;
	}

	/**
	 * Sets the Dimensions property for Regional Map.
	 * The size of each dimension.
	 * @param _regionalMapDimensions the new Dimensions value
	 */
	public void setRegionalMapDimensions(int[] _regionalMapDimensions) {
		regionalMapDimensions = _regionalMapDimensions;
	}

	/**
	 * Gets the Width property for Regional Map.
	 * @return The horizontal extent of the space.
	 */
	public int getRegionalMapWidth() {
		return regionalMapWidth;
	}

	/**
	 * Sets the Width property for Regional Map.
	 * The horizontal extent of the space.
	 * @param _regionalMapWidth the new Width value
	 */
	public void setRegionalMapWidth(int _regionalMapWidth) {
		regionalMapWidth = _regionalMapWidth;
	}

	/**
	 * Gets the Height property for Regional Map.
	 * @return The vertical extent of the space.
	 */
	public int getRegionalMapHeight() {
		return regionalMapHeight;
	}

	/**
	 * Sets the Height property for Regional Map.
	 * The vertical extent of the space.
	 * @param _regionalMapHeight the new Height value
	 */
	public void setRegionalMapHeight(int _regionalMapHeight) {
		regionalMapHeight = _regionalMapHeight;
	}

	/**
	 * Gets the Regional Map property for Region.
	 * @return 
	 */
	public Scape getRegionalMap() {
		return regionalMap;
	}

	/**
	 * Sets the Regional Map property for Region.
	 * 
	 * @param _regionalMap the new Regional Map value
	 */
	public void setRegionalMap(Scape _regionalMap) {
		regionalMap = _regionalMap;
	}

	public String getName() {
		if (name == null) {
			return "Region " + getUID();
		} else {
			return name;
		}
	}
}
