package name.milesparker.markets;

import java.util.List;

import org.eclipse.core.runtime.IAdapterFactory;
import org.eclipse.amp.agf.IGraphicsAdapted;
import org.eclipse.amp.agf.IGraphicsAdapter;

import org.eclipse.swt.graphics.Color;
import org.eclipse.jface.viewers.IColorProvider;

import org.ascape.model.Agent;
import org.ascape.model.Cell;
import org.ascape.model.CellOccupant;
import org.ascape.model.HostCell;
import org.ascape.model.LocatedAgent;
import org.ascape.model.Scape;
import org.ascape.model.event.ScapeEvent;
import org.ascape.model.rule.Rule;
import org.ascape.model.rule.ExecuteThenUpdate;
import org.ascape.model.space.CollectionSpace;
import org.ascape.model.space.Coordinate;
import org.ascape.model.space.Coordinate2DDiscrete;
import org.ascape.model.space.Graph;
import org.ascape.model.space.Location;
import org.ascape.runtime.NonGraphicRunner;
import org.ascape.util.Conditional;
import org.ascape.util.data.DataPoint;
import org.ascape.util.data.DataPointConcrete;
import org.ascape.util.vis.ColorFeature;
import org.ascape.util.vis.ColorFeatureConcrete;
import org.ascape.view.vis.ChartView;
import org.ascape.view.vis.GEFView;
import org.ascape.view.vis.GraphView;

/**
 * Trader Java Implementation.
 * 
 * Generated by AMF for model: Markets.metaabm in project: org.eclipse.amp.amf.examples.escape 
 */
public class Trader extends CellOccupant {

	/**
	 * 
	 */
	private StrategyEnum strategy = StrategyEnum.random;

	/**
	 * 
	 */
	private double funds = 100000;

	/**
	 * 
	 */
	private int instrumentCount = 0;

	/**
	 * Constructs a new Trader.
	 */
	public Trader() {
	}
	//todo, make this a useful value for evaluating compatibility of different versions of generated classes
	private static final long serialVersionUID = 89989998L;

	private static long nextUniqueID;

	private long uniqueID;

	public long getUID() {
		if (uniqueID == 0) {
			uniqueID = nextUniqueID++;
		}
		return uniqueID;
	}

	/**
	 * Clones the agent, ensuring that a unique id is assigned.
	 */
	public Object clone() {
		try {
			Trader clone = (Trader) super.clone();
			clone.uniqueID = 0;
			return clone;
		} catch (Exception e) {
			throw new RuntimeException("Unexpected cloning exception: " + e);
		}
	}

	public Markets getMarkets() {
		return (Markets) getScape().getScape();
	}
	public java.util.List getNetwork() {
		return ((Graph) getMarkets().getInstrumentHolders().getSpace())
				.getNeighborsFor(this);
	}

	/**
	 * Initialize Initialization. Executed once at the beginning of each model run.
	 */
	public void intializeNonFramework() {
		StrategyEnum randomState_Strategy_ = StrategyEnum.values()[randomToLimit(StrategyEnum
				.values().length)];
		double initialTraderFunds = initialTraderFunds();
		setStrategy(randomState_Strategy_);
		setFunds(initialTraderFunds);
	}

	/**
	 * Ask Rule. Executed every period.
	 */
	public void ask() {
		Conditional sellingCondition = new Conditional() {
			private static final long serialVersionUID = 6846144446402098985L;
			public boolean meetsCondition(Object sellingCell) {
				if (sellingCell instanceof Instrument) {
					Instrument selling = (Instrument) sellingCell;
					return (selling.getStatus() == StatusEnum.held);
				} else {
					return false;
				}
			}
		};
		Instrument selling = (Instrument) ((org.ascape.model.space.Discrete) getMarkets()
				.getInstrumentHolders().getSpace()).findRandomNeighbor(this,
				sellingCondition);
		if (selling != null) {
			Conditional instrumentFirmCondition = new Conditional() {
				private static final long serialVersionUID = 6846144446402098985L;
				public boolean meetsCondition(Object instrumentFirmCell) {
					if (instrumentFirmCell instanceof Firm) {
						return true;
					} else {
						return false;
					}
				}
			};
			Firm instrumentFirm = (Firm) ((org.ascape.model.space.Discrete) selling
					.getMarkets().getFirmInstruments().getSpace())
					.findRandomNeighbor(selling, instrumentFirmCondition);
			if (instrumentFirm != null) {
				double sharePriceAdjustment = sharePriceAdjustment();
				double newSharePrice = sharePriceAdjustment
						* instrumentFirm.getLastPrice();
				selling.setOfferPrice(newSharePrice);
				selling.setStatus(StatusEnum.ask);
			}
		}
	}

	/**
	 * Buy Rule. Executed every period.
	 */
	public void buy() {

		((org.ascape.model.space.Graph) getMarkets().getTargetBuyFirms()
				.getSpace()).clearNeighbors(this);
		Firm offeredFirm = (Firm) getMarkets().getFirmScape().findRandom();
		if (offeredFirm != null) {
			((org.ascape.model.space.Graph) getMarkets().getTargetBuyFirms()
					.getSpace()).addNeighborSafe(this, offeredFirm, false);
		}
	}

	/**
	 * Calculate Share Price Adjustment.
	 */
	public double sharePriceAdjustment() {
		double sharePriceAdjustmentMinimum = getMarkets()
				.getSharePriceAdjustmentMinimum();
		double sharePriceAdjustmentMaximum = getMarkets()
				.getSharePriceAdjustmentMaximum();
		return randomInRange(sharePriceAdjustmentMinimum,
				sharePriceAdjustmentMaximum);
	}

	/**
	 * Calculate Initial Trader Funds.
	 */
	public double initialTraderFunds() {
		double initialTraderFundsMinimum = getMarkets()
				.getInitialTraderFundsMinimum();
		double initialTraderFundsMaximum = getMarkets()
				.getInitialTraderFundsMaximum();
		return randomInRange(initialTraderFundsMinimum,
				initialTraderFundsMaximum);
	}

	/**
	 * Execute Rule. Executed every period.
	 */
	public void execute() {
		Conditional firmCondition = new Conditional() {
			private static final long serialVersionUID = 6846144446402098985L;
			public boolean meetsCondition(Object firmCell) {
				if (firmCell instanceof Firm) {
					return true;
				} else {
					return false;
				}
			}
		};
		Firm firm = (Firm) ((org.ascape.model.space.Discrete) getMarkets()
				.getTargetBuyFirms().getSpace()).findRandomNeighbor(this,
				firmCondition);
		if (firm != null) {
			Conditional boughtInstrumentCondition = new Conditional() {
				private static final long serialVersionUID = 6846144446402098985L;
				public boolean meetsCondition(Object boughtInstrumentCell) {
					if (boughtInstrumentCell instanceof Instrument) {
						Instrument boughtInstrument = (Instrument) boughtInstrumentCell;
						return (boughtInstrument.getStatus() == StatusEnum.ask && boughtInstrument
								.getTotalPrice() <= getFunds());
					} else {
						return false;
					}
				}
			};
			Instrument boughtInstrument = (Instrument) ((org.ascape.model.space.Discrete) firm
					.getMarkets().getFirmInstruments().getSpace())
					.findRandomNeighbor(firm, boughtInstrumentCondition);
			if (boughtInstrument != null) {
				boughtInstrument.setStatus(StatusEnum.held);
				double fundsSubtractTotalPrice = getFunds()
						- boughtInstrument.getTotalPrice();
				setFunds(fundsSubtractTotalPrice);
				double offerPriceAddZero = boughtInstrument.getOfferPrice() + 0;
				firm.setLastPrice(offerPriceAddZero);
				Conditional sellerCondition = new Conditional() {
					private static final long serialVersionUID = 6846144446402098985L;
					public boolean meetsCondition(Object sellerCell) {
						if (sellerCell instanceof Trader) {
							return true;
						} else {
							return false;
						}
					}
				};
				Trader seller = (Trader) ((org.ascape.model.space.Discrete) getMarkets()
						.getInstrumentHolders().getSpace()).findRandomNeighbor(
						boughtInstrument, sellerCondition);
				if (seller != null) {
					double fundsAddTotalPrice = seller.getFunds()
							+ boughtInstrument.getTotalPrice();
					seller.setFunds(fundsAddTotalPrice);
					((org.ascape.model.space.Graph) getMarkets()
							.getInstrumentHolders().getSpace())
							.clearNeighbors(seller);
					((org.ascape.model.space.Graph) getMarkets()
							.getInstrumentHolders().getSpace())
							.addNeighborSafe(this, boughtInstrument, false);
					int incrementCount = getInstrumentCount() + 1;
					setInstrumentCount(incrementCount);
					int decrementSellerCount = seller.getInstrumentCount() - 1;
					seller.setInstrumentCount(decrementSellerCount);
				}
			}
		}
	}
	/**
	 * Gets the Strategy property for Trader.
	 * @return 
	 */
	public StrategyEnum getStrategy() {
		return strategy;
	}

	/**
	 * Sets the Strategy property for Trader.
	 * 
	 * @param _strategy the new Strategy value
	 */
	public void setStrategy(StrategyEnum _strategy) {
		strategy = _strategy;
	}

	/**
	 * Gets the Funds property for Trader.
	 * @return 
	 */
	public double getFunds() {
		return funds;
	}

	/**
	 * Sets the Funds property for Trader.
	 * 
	 * @param _funds the new Funds value
	 */
	public void setFunds(double _funds) {
		funds = _funds;
	}

	/**
	 * Gets the Instrument Count property for Trader.
	 * @return 
	 */
	public int getInstrumentCount() {
		return instrumentCount;
	}

	/**
	 * Sets the Instrument Count property for Trader.
	 * 
	 * @param _instrumentCount the new Instrument Count value
	 */
	public void setInstrumentCount(int _instrumentCount) {
		instrumentCount = _instrumentCount;
	}

	public String getName() {
		if (name == null) {
			return "Trader " + getUID();
		} else {
			return name;
		}
	}
}
