package org.eclipse.amp.examples.heatbugs;

import java.util.List;

import org.eclipse.core.runtime.IAdapterFactory;
import org.eclipse.amp.agf.IGraphicsAdapted;
import org.eclipse.amp.agf.IGraphicsAdapter;

import org.eclipse.swt.graphics.Color;
import org.eclipse.jface.viewers.IColorProvider;

import org.ascape.model.Agent;
import org.ascape.model.Cell;
import org.ascape.model.CellOccupant;
import org.ascape.model.HostCell;
import org.ascape.model.LocatedAgent;
import org.ascape.model.Scape;
import org.ascape.model.event.ScapeEvent;
import org.ascape.model.rule.Rule;
import org.ascape.model.rule.ExecuteThenUpdate;
import org.ascape.model.space.CollectionSpace;
import org.ascape.model.space.Coordinate;
import org.ascape.model.space.Coordinate2DDiscrete;
import org.ascape.model.space.Graph;
import org.ascape.model.space.Location;
import org.ascape.runtime.NonGraphicRunner;
import org.ascape.util.Conditional;
import org.ascape.util.data.DataPoint;
import org.ascape.util.data.DataPointConcrete;
import org.ascape.util.vis.ColorFeature;
import org.ascape.util.vis.ColorFeatureConcrete;
import org.ascape.view.vis.ChartView;
import org.ascape.view.vis.GEFView;
import org.ascape.view.vis.GraphView;

/**
 * Heat Cell Java Implementation.
 * 
 * Generated by AMF for model: Heatbugs.metaabm in project: org.eclipse.amp.amf.examples.escape 
 */
public class HeatCell extends HostCell {

	/**
	 * 
	 */
	private double heat = 0.0;

	/**
	 * Constructs a new Heat Cell.
	 */
	public HeatCell() {
	}
	//todo, make this a useful value for evaluating compatibility of different versions of generated classes
	private static final long serialVersionUID = 89989998L;

	private static long nextUniqueID;

	private long uniqueID;

	public long getUID() {
		if (uniqueID == 0) {
			uniqueID = nextUniqueID++;
		}
		return uniqueID;
	}

	/**
	 * Clones the agent, ensuring that a unique id is assigned.
	 */
	public Object clone() {
		try {
			HeatCell clone = (HeatCell) super.clone();
			clone.uniqueID = 0;
			return clone;
		} catch (Exception e) {
			throw new RuntimeException("Unexpected cloning exception: " + e);
		}
	}

	public Heatbugs getHeatbugs() {
		return (Heatbugs) getScape().getScape();
	}

	private double diffuseTemp;

	public static final Rule DIFFUSE_RULE = new ExecuteThenUpdate("Diffuse") {
		private static final long serialVersionUID = 6846144446402098983L;
		/**
		 * Calculate and store diffusion value to neighbors.
		 * 
		 * @param agent
		 *            the playing agent
		 */
		public void execute(Agent agent) {
			HeatCell heatCell = (HeatCell) agent;
			double valueSum = 0.0f;
			List neighbors = ((Cell) agent).findNeighbors();
			for (Object neighbor : neighbors) {
				HeatCell heatCellNeighbor = (HeatCell) neighbor;
				valueSum += heatCellNeighbor.getHeat();
			}
			valueSum -= neighbors.size() * heatCell.getHeat();
			valueSum /= neighbors.size();
			valueSum = heatCell.getHeat() + valueSum
					* heatCell.getHeatbugs().getDiffusionRate();
			valueSum *= 1.0f - heatCell.getHeatbugs().getEvaporationRate();
			heatCell.diffuseTemp = valueSum;
		}
		/**
		 * Update diffusion value. Occurs only when all diffusion has been
		 * calculated.
		 * 
		 * @param agent
		 *            the agent
		 */
		public void update(Agent agent) {
			HeatCell heatCell = (HeatCell) agent;
			heatCell.setHeat(heatCell.diffuseTemp);
		}
	};

	/**
	 * Watch for changes in Heat.
	 */
	public void watchHeat() {
		if (getHeat() > getHeatbugs().getMaximumHeat()) {
			setHeat(getHeatbugs().getMaximumHeat());
		}
	}
	/**
	 * Gets the Heat property for Heat Cell.
	 * @return 
	 */
	public double getHeat() {
		return heat;
	}

	/**
	 * Sets the Heat property for Heat Cell.
	 * 
	 * @param _heat the new Heat value
	 */
	public void setHeat(double _heat) {
		heat = _heat;
		watchHeat();
	}

	public String getName() {
		if (name == null) {
			return "Heat Cell " + getUID();
		} else {
			return name;
		}
	}
}
