package org.eclipse.amp.examples.heatbugs;

import java.util.List;

import org.eclipse.core.runtime.IAdapterFactory;
import org.eclipse.amp.agf.IGraphicsAdapted;
import org.eclipse.amp.agf.IGraphicsAdapter;

import org.eclipse.swt.graphics.Color;
import org.eclipse.jface.viewers.IColorProvider;

import org.ascape.model.Agent;
import org.ascape.model.Cell;
import org.ascape.model.CellOccupant;
import org.ascape.model.HostCell;
import org.ascape.model.LocatedAgent;
import org.ascape.model.Scape;
import org.ascape.model.event.ScapeEvent;
import org.ascape.model.rule.Rule;
import org.ascape.model.rule.ExecuteThenUpdate;
import org.ascape.model.space.CollectionSpace;
import org.ascape.model.space.Coordinate;
import org.ascape.model.space.Coordinate2DDiscrete;
import org.ascape.model.space.Graph;
import org.ascape.model.space.Location;
import org.ascape.runtime.NonGraphicRunner;
import org.ascape.util.Conditional;
import org.ascape.util.data.DataPoint;
import org.ascape.util.data.DataPointConcrete;
import org.ascape.util.vis.ColorFeature;
import org.ascape.util.vis.ColorFeatureConcrete;
import org.ascape.view.vis.ChartView;
import org.ascape.view.vis.GEFView;
import org.ascape.view.vis.GraphView;

/**
 * Heatbugs Java Implementation.
 * 
 * Generated by AMF for model: Heatbugs.metaabm in project: org.eclipse.amp.amf.examples.escape 
 */
public class Heatbugs extends Scape implements IGraphicsAdapted {

	/**
	 * The number of heat bugs to create.
	 */
	private int heatBugCount = 100;

	/**
	 * 
	 */
	private double diffusionRate = 1.0;

	/**
	 * 
	 */
	private double evaporationRate = .01;

	/**
	 * 
	 */
	private double idealTemperatureDrawMinimum = 135;

	/**
	 * 
	 */
	private double idealTemperatureDrawMaximum = 240;

	/**
	 * 
	 */
	private double outputHeatDrawMinimum = 23;

	/**
	 * 
	 */
	private double outputHeatDrawMaximum = 76;

	/**
	 * 
	 */
	private int maximumHeat = 340;

	/**
	 * The horizontal extent of the space.
	 */
	private int heatGridWidth = 100;

	/**
	 * The vertical extent of the space.
	 */
	private int heatGridHeight = 100;

	/**
	 * 
	 */
	private Scape heatGrid = null;

	/**
	 * Constructs a new Heatbugs.
	 */
	public Heatbugs() {
	}

	//todo, make this a useful value for evaluating compatibility of different versions of generated classes
	private static final long serialVersionUID = 89989998L;

	private static long nextUniqueID;

	private long uniqueID;

	public long getUID() {
		if (uniqueID == 0) {
			uniqueID = nextUniqueID++;
		}
		return uniqueID;
	}

	/**
	 * Clones the agent, ensuring that a unique id is assigned.
	 */
	public Object clone() {
		try {
			Heatbugs clone = (Heatbugs) super.clone();
			clone.uniqueID = 0;
			return clone;
		} catch (Exception e) {
			throw new RuntimeException("Unexpected cloning exception: " + e);
		}
	}

	org.ascape.model.Scape heatBugScape;

	/**
	 * Create an instance of Heat Bug that will be used to populate the heatBugScape.
	 * Overide to customize the prototype, for example to change the views created for a member scape.
	 */
	protected HeatBug createHeatBugPrototype() {
		return new HeatBug();
	}

	static int next_vm_id;

	int vm_unique_id;

	/**
	 * Creates the members of Heatbugs. 
	 */
	public void createScape() {
		super.createScape();

		setName("Heatbugs");
		HeatBug heatBugProto = createHeatBugPrototype();
		heatBugScape = new Scape();
		heatBugScape.setName("Heat Bugs");
		heatBugScape.setPrototypeAgent(heatBugProto);
		heatBugScape.setExecutionOrder(Scape.RULE_ORDER);

		heatBugScape.setSize(heatBugCount);
		heatGrid = new Scape(new org.ascape.model.space.Array2DMoore());
		heatGrid.setExecutionOrder(Scape.RULE_ORDER);
		heatGrid.setPrototypeAgent(new HeatCell());
		heatGrid.setExtent(new org.ascape.model.space.Coordinate2DDiscrete(
				getHeatGridWidth(), getHeatGridHeight()));

		heatGrid.setName("Heat Grids");
		((CollectionSpace) heatGrid.getSpace()).setPeriodic(false);
		add(heatGrid);
		org.ascape.model.Scape heatCellScape = heatGrid;

		add(heatBugScape);

		heatBugProto.setHostScape(heatGrid);

		heatBugScape
				.addStatCollector(new org.ascape.util.data.StatCollectorCond(
						getName() + " Population") {
					private static final long serialVersionUID = 684614444640209893L;

					@SuppressWarnings("unused")
					public final boolean meetsCondition(Object object) {
						return true;
					}
				});
		heatBugScape
				.addStatCollector(new org.ascape.util.data.StatCollectorCSAMM(
						"Heat Bug Unhappiness") {
					private static final long serialVersionUID = 6846144446402098985L;
					public final double getValue(Object heatBug) {
						return ((HeatBug) heatBug).getUnhappiness();
					}
				});

		heatBugScape.addInitialRule(new Rule("Initialize") {
			private static final long serialVersionUID = 6846144446402098982L;
			public void execute(Agent a) {
				((org.eclipse.amp.examples.heatbugs.HeatBug) a)
						.intializeNonFramework();
			}
		});
		heatBugScape.addRule(new Rule("Seek Heat Rule") {
			private static final long serialVersionUID = 6846144446402098981L;
			public void execute(Agent a) {
				((org.eclipse.amp.examples.heatbugs.HeatBug) a).seekHeatRule();
			}
		});

		heatBugScape.addInitialRule(new Rule("Initialize State") {
			private static final long serialVersionUID = 6846144446402098982L;
			public void execute(Agent a) {
				((org.eclipse.amp.examples.heatbugs.HeatBug) a)
						.initializeState();
			}
		});
		heatCellScape.addRule(HeatCell.DIFFUSE_RULE);

	}

	/**
	 * Creates UI views for Heatbugs.
	 */
	public void createGraphicViews() {
		super.createGraphicViews();
		ChartView chart = new ChartView();

		final Heatbugs heatBugScape = new Heatbugs();
		heatBugScape.setRunner(new NonGraphicRunner());
		HeatBug heatBug = new HeatBug() {
			public void requestUpdate() {
			}
			public Heatbugs getHeatbugs() {
				return heatBugScape;
			}
		};
		heatBugScape.add(heatBug);
		HostCell heatBugHost = new HostCell() {
			public void requestUpdate() {
			}
		};
		heatBugHost.setOccupant((CellOccupant) heatBug);

		IColorProvider heatBugStyle2DColorProvider = new HeatBugStyle2DColorProvider();

		final Heatbugs heatCellScape = new Heatbugs();
		heatCellScape.setRunner(new NonGraphicRunner());
		HeatCell heatCell = new HeatCell() {
			public void requestUpdate() {
			}
			public Heatbugs getHeatbugs() {
				return heatCellScape;
			}
		};
		heatCellScape.add(heatCell);

		IColorProvider heatCellStyle2DColorProvider = new HeatCellStyle2DColorProvider();
		addView(chart);
		addView(new GEFView());
	}

	public void scapeSetup(ScapeEvent scapeEvent) {
		//Set the size of the scape populations, e.g. for mutable scapes that need to have their size reset

		heatBugScape.setSize(heatBugCount);
	}

	/**
	 * Returns the Scape containing Heat Bugs.
	 */
	public org.ascape.model.Scape getHeatBugScape() {
		return heatBugScape;
	}

	public IGraphicsAdapter getGraphicsAdapter() {
		return HeatbugsGraphicsAdapter.getDefault();
	}

	private Color getInferredChartColor(Object agent,
			IColorProvider colorProvider) {
		Color color = colorProvider.getForeground(agent);
		if (color == ColorFeature.BLACK) {
			color = ColorFeatureConcrete.createHSB(
					(float) java.lang.Math.random() * 360f, 1.0f, 0.8f);
		}
		return color;
	}

	public static void main(String[] args) {
		(new NonGraphicRunner())
				.openInstance("org.eclipse.amp.examples.heatbugs.Heatbugs");
	}
	/**
	 * Gets the Heat Bug Count property for Heatbugs.
	 * @return The number of heat bugs to create.
	 */
	public int getHeatBugCount() {
		return heatBugCount;
	}

	/**
	 * Sets the Heat Bug Count property for Heatbugs.
	 * The number of heat bugs to create.
	 * @param _heatBugCount the new Heat Bug Count value
	 */
	public void setHeatBugCount(int _heatBugCount) {
		heatBugCount = _heatBugCount;
	}

	/**
	 * Gets the Diffusion Rate property for Heatbugs.
	 * @return 
	 */
	public double getDiffusionRate() {
		return diffusionRate;
	}

	/**
	 * Sets the Diffusion Rate property for Heatbugs.
	 * 
	 * @param _diffusionRate the new Diffusion Rate value
	 */
	public void setDiffusionRate(double _diffusionRate) {
		diffusionRate = _diffusionRate;
	}

	/**
	 * Gets the Evaporation Rate property for Heatbugs.
	 * @return 
	 */
	public double getEvaporationRate() {
		return evaporationRate;
	}

	/**
	 * Sets the Evaporation Rate property for Heatbugs.
	 * 
	 * @param _evaporationRate the new Evaporation Rate value
	 */
	public void setEvaporationRate(double _evaporationRate) {
		evaporationRate = _evaporationRate;
	}

	/**
	 * Gets the Ideal Temperature Draw Minimum property for Heatbugs.
	 * @return 
	 */
	public double getIdealTemperatureDrawMinimum() {
		return idealTemperatureDrawMinimum;
	}

	/**
	 * Sets the Ideal Temperature Draw Minimum property for Heatbugs.
	 * 
	 * @param _idealTemperatureDrawMinimum the new Ideal Temperature Draw Minimum value
	 */
	public void setIdealTemperatureDrawMinimum(
			double _idealTemperatureDrawMinimum) {
		idealTemperatureDrawMinimum = _idealTemperatureDrawMinimum;
	}

	/**
	 * Gets the Ideal Temperature Draw Maximum property for Heatbugs.
	 * @return 
	 */
	public double getIdealTemperatureDrawMaximum() {
		return idealTemperatureDrawMaximum;
	}

	/**
	 * Sets the Ideal Temperature Draw Maximum property for Heatbugs.
	 * 
	 * @param _idealTemperatureDrawMaximum the new Ideal Temperature Draw Maximum value
	 */
	public void setIdealTemperatureDrawMaximum(
			double _idealTemperatureDrawMaximum) {
		idealTemperatureDrawMaximum = _idealTemperatureDrawMaximum;
	}

	/**
	 * Gets the Output Heat Draw Minimum property for Heatbugs.
	 * @return 
	 */
	public double getOutputHeatDrawMinimum() {
		return outputHeatDrawMinimum;
	}

	/**
	 * Sets the Output Heat Draw Minimum property for Heatbugs.
	 * 
	 * @param _outputHeatDrawMinimum the new Output Heat Draw Minimum value
	 */
	public void setOutputHeatDrawMinimum(double _outputHeatDrawMinimum) {
		outputHeatDrawMinimum = _outputHeatDrawMinimum;
	}

	/**
	 * Gets the Output Heat Draw Maximum property for Heatbugs.
	 * @return 
	 */
	public double getOutputHeatDrawMaximum() {
		return outputHeatDrawMaximum;
	}

	/**
	 * Sets the Output Heat Draw Maximum property for Heatbugs.
	 * 
	 * @param _outputHeatDrawMaximum the new Output Heat Draw Maximum value
	 */
	public void setOutputHeatDrawMaximum(double _outputHeatDrawMaximum) {
		outputHeatDrawMaximum = _outputHeatDrawMaximum;
	}

	/**
	 * Gets the Maximum Heat property for Heatbugs.
	 * @return 
	 */
	public int getMaximumHeat() {
		return maximumHeat;
	}

	/**
	 * Sets the Maximum Heat property for Heatbugs.
	 * 
	 * @param _maximumHeat the new Maximum Heat value
	 */
	public void setMaximumHeat(int _maximumHeat) {
		maximumHeat = _maximumHeat;
	}

	/**
	 * Gets the Width property for Heat Grid.
	 * @return The horizontal extent of the space.
	 */
	public int getHeatGridWidth() {
		return heatGridWidth;
	}

	/**
	 * Sets the Width property for Heat Grid.
	 * The horizontal extent of the space.
	 * @param _heatGridWidth the new Width value
	 */
	public void setHeatGridWidth(int _heatGridWidth) {
		heatGridWidth = _heatGridWidth;
	}

	/**
	 * Gets the Height property for Heat Grid.
	 * @return The vertical extent of the space.
	 */
	public int getHeatGridHeight() {
		return heatGridHeight;
	}

	/**
	 * Sets the Height property for Heat Grid.
	 * The vertical extent of the space.
	 * @param _heatGridHeight the new Height value
	 */
	public void setHeatGridHeight(int _heatGridHeight) {
		heatGridHeight = _heatGridHeight;
	}

	/**
	 * Gets the Heat Grid property for Heatbugs.
	 * @return 
	 */
	public Scape getHeatGrid() {
		return heatGrid;
	}

	/**
	 * Sets the Heat Grid property for Heatbugs.
	 * 
	 * @param _heatGrid the new Heat Grid value
	 */
	public void setHeatGrid(Scape _heatGrid) {
		heatGrid = _heatGrid;
	}

	public String getName() {
		if (name == null) {
			return "Heatbugs " + getUID();
		} else {
			return name;
		}
	}
}
